// Includes (yes, I really have to comment every block of code I write.)
#include <sourcemod>
#include <sdktools>
#include <tf2>
#include <tf2_stocks>

// Constants
#define PLUGIN_VERSION "1.0.0"
#define MAX_WEAPON_LENGTH 128

// Information about the plugin
public Plugin:myinfo =
{
	name = "[TF2] Airshot Bride",
	author = "Jerry",
	description = "Plays a sound upon a successful airshot.",
	version = PLUGIN_VERSION,
	url = "http://jerry.devzero.info/"
}

// CVars
new Handle:AirshotSound = INVALID_HANDLE;
new Handle:AirshotVolume = INVALID_HANDLE;
new Handle:AirshotStopSound = INVALID_HANDLE;
new Handle:AirshotHeadshotOnly = INVALID_HANDLE;

// Weapons that can be used to trigger the airshot sound
new String:AirshotWeapons[][] =
{
	// Rocket Launchers
	"tf_projectile_rocket",             // Rocket Launcher
	"rocketlauncher_directhit",
	"blackbox",
	"liberty_launcher",
	"quake_rl",                         // Original
	"dumpster_device",                  // Beggar's Bazooka
	"cow_mangler",

	// Grenade Launchers
	"tf_projectile_pipe",               // Grenade Launcher
	"loch_n_load",

	// Stickybomb Launchers
	"tf_projectile_pipe_remote",        // Stickybomb Launcher
	"sticky_resistance",                // Scottish Resistance

	// Sniper Rifles
	"sniperrifle",
	"huntsman",
	"sydney_sleeper",
	"bazaar_bargain",
	"machina",
	"pro_rifle",                        // Hitman's Heatmaker

	// Revolvers
	"revolver",
	"samrevolver",                      // Big Kill
	"ambassador",
	"letranger",
	"enforcer",
	"diamondback",

	// Fun stuff
	"market_gardener"
}

// Weapons that must headshot in headshot-only-mode (including weapons that can't headshot)
new String:HeadshotWeapons[][] =
{
	"sniperrifle",
	"huntsman",
	"sydney_sleeper",
	"bazaar_bargain",
	"machina",
	"pro_rifle",                        // Hitman's Heatmaker

	"revolver",
	"samrevolver",                      // Big Kill
	"ambassador",
	"letranger",
	"enforcer",
	"diamondback"
}

// Plugin initialization
public OnPluginStart()
{
	// Hook the player_death event
	HookEvent("player_death", EventPlayerDeath);

	// Create CVars and have SourceMod automatically create the configuration file for us
	AirshotSound = CreateConVar("sm_airshot_sound", "misc/airshot_sound.mp3", "Path to the sound file that is played when a player dies to an airshot");
	AirshotVolume = CreateConVar("sm_airshot_volume", "1.0", "Volume of the airshot sound (0.0 = completely muted, 1.0 = full volume)", 0, true, 0.0, true, 1.0);
	AirshotStopSound = CreateConVar("sm_airshot_stopsound", "1", "If set to 1, the airshot sound will be restarted if it's still playing while another airshot occurs.");
	AirshotHeadshotOnly = CreateConVar("sm_airshot_headshot_only", "1", "If set to 1, players using any Sniper Rifle or Revolver *will* have to do headshots for the airshot sound to play.");
	AutoExecConfig(true, "plugin.airshot-bride");
}

// Map initialization
public OnMapStart()
{
	new String:SoundFile[PLATFORM_MAX_PATH];
	new String:FullSoundFile[PLATFORM_MAX_PATH];
	
	// Precache the sound file
	GetConVarString(AirshotSound, SoundFile, sizeof(SoundFile));
	PrecacheSound(SoundFile);

	// Prepend "sounds/" and add it to the downloads table
	Format(FullSoundFile, sizeof(FullSoundFile), "sound/%s", SoundFile);
	AddFileToDownloadsTable(FullSoundFile);
}

// Player death (main event)
public Action:EventPlayerDeath(Handle:event, const String:name[], bool:dontBroadcast)
{
	// Client and attacker
	new client = GetClientOfUserId(GetEventInt(event, "userid"));
	new attacker = GetClientOfUserId(GetEventInt(event, "attacker"));

	// Weapon stuff
	new String:weapon[MAX_WEAPON_LENGTH];
	new customkill = GetEventInt(event, "customkill");
	new airshot_weapons = sizeof(AirshotWeapons);
	new headshot_weapons = sizeof(HeadshotWeapons);
	new bool:can_headshot;
	new bool:weapon_eligible;

	// Sound stuff
	new String:SoundFile[PLATFORM_MAX_PATH];
	new Float:volume;
	new bool:play_sound;

	// Loop variable
	new i;

	// Check for *valid* client and attacker
	if ((client < 1) || (client > MaxClients) || (attacker < 1) || (attacker > MaxClients))
	{
		// Probably an environmental death, let's just ignore it.
		return Plugin_Continue;
	}

	// Alright, get the name of the used weapon
	GetEventString(event, "weapon", weapon, sizeof(weapon));

	// Killed by world? Better ignore it.
	if (StrEqual(weapon, "world"))
		return Plugin_Continue;
	
	// If they are not in the water and not in the air, they are airborne. And if they're not airborne, it's not an airshot.
	if ((GetEntityFlags(client) & (FL_INWATER | FL_ONGROUND)))
		return Plugin_Continue;

	// Check whether this weapon can be used to trigger the airshot sound
	for (i = 0; i < airshot_weapons; ++i)
	{
		if (StrEqual(weapon, AirshotWeapons[i]))
		{
			weapon_eligible = true;
			break;
		}
	}

	// Only continue if the weapon cannot trigger airshots.
	if (!weapon_eligible)
		return Plugin_Continue;

	// Do we need to check for headshots?
	if (GetConVarBool(AirshotHeadshotOnly))
	{
		// Headshot-only-mode is enabled, check whether this weapon can do headshots.
		for (i = 0; i < headshot_weapons; ++i)
		{
			if (StrEqual(weapon, HeadshotWeapons[i]))
			{
				can_headshot = true;
				break;
			}
		}

		if (can_headshot)
		{
			// It can, or rather, must do headshots to play the airshot sound.
			if ((customkill == TF_CUSTOM_HEADSHOT) || (customkill == TF_CUSTOM_HEADSHOT_DECAPITATION) || (customkill == TF_CUSTOM_PENETRATE_HEADSHOT))
			{
				// It's a headshot, play the sound.
				play_sound = true;
			}
		}
		else
		{
			// It can't headshot, so let's just play the sound.
			play_sound = true;
		}
	}
	else
	{
		// Headshot-only-mode is disabled; we already checked whether it's an actual airshot, so let's play the sound.
		play_sound = true;
	}

	// Return if the sound should not be played.
	if (!play_sound)
		return Plugin_Continue;

	// Get the sound file and the volume
	GetConVarString(AirshotSound, SoundFile, sizeof(SoundFile));
	volume = GetConVarFloat(AirshotVolume);

	// Stop the sound first if necessary
	if (GetConVarBool(AirshotStopSound))
		EmitSoundToAll(SoundFile, _, _, _, SND_STOPLOOPING);

	// Play the sound
	EmitSoundToAll(SoundFile, _, _, _, _, volume);

	return Plugin_Continue;
}
